function DemonstrationFeatures()
%*************************************************************************
%*                                                                       *
%*  Demonstration file of objects features detection in an image. At     *
%*  first there is necessary to set up parameters of a camera in your    *
%*  computer in section "camera setting" below.                          *
%*                                                                       *
%*  Author: Karel Horak, Brno University of Technology, 2012             *
%*                                                                       *
%*************************************************************************

%% cleaning
close all;
clear;
clc;

%% camera settings
% camera object
imaqreset;
cam = videoinput('winvideo',1,'RGB24_320x240');
cam.ReturnedColorSpace = 'grayscale';
triggerconfig(cam,'manual');
ImgSize   = get(cam,'VideoResolution');
ImgPixels = ImgSize(1)*ImgSize(2);
% camera parameters
params                  = getselectedsource(cam);
params.ExposureMode     = 'manual';
params.Exposure         = -8;
params.WhiteBalanceMode = 'manual';

%% parameters
global AppRunning
AppRunning    = 1;
ImgCnt        = 0;  % image counter
TextColor     = [1 1 0];
DrawColor     = [1 0 0];
FontSize      = 16;
NoiseCnt      = round(ImgPixels/1000); % noise size
se            = strel('disk',3); %structure element for morphology
MotionThr     = 2;
DifferenceThr = 3;
CircleValThr  = 16;

%% GUI
ScreenSize = get(0,'ScreenSize');
figure('Position',[10 1 ScreenSize(3)-20 ScreenSize(4)-70],'Color',[0.2 0.2 0.2],'KeyPressFcn',@MyKeyPressed);
hold on;

%% background model
ModelNumImages = 20; % number of images to create the model
img_model = zeros(ImgSize(2),ImgSize(1),'uint16');
start(cam);
for ImgCnt = 1:ModelNumImages
    img = getsnapshot(cam);
    img_model = img_model + uint16(img);
end;
stop(cam);
img_model = uint8( img_model/ModelNumImages );

%% feature detection main loop
start(cam);
tic;
while (AppRunning == 1)
    %% image acquisition
    img_old = img;
    img = getsnapshot(cam);
    ImgCnt = ImgCnt + 1;
    
    %% motion detection
    img_motion = imabsdiff(img,img_old);           % motion between two subsequent images
    sum_motion = sum(sum(img_motion)) / ImgPixels; % relative motion of one pixel
    
    %% differential image
    img_diff = imabsdiff(img,img_model);       % difference between current image and model
    sum_diff = sum(sum(img_diff)) / ImgPixels; % relative difference of one pixel
    
    %% object detection by differential image in static scene
    if ( sum_motion<MotionThr && sum_diff>DifferenceThr ) 
        %% static scene with some object(s)
        ObjectDetected = 1;

        %% image filtering
        ThrLevel = 0.5*graythresh(img_diff); % graythresh to binary image threshold
        img_bw   = im2bw(img_diff, ThrLevel);
        img_seg  = bwareaopen(img_bw, NoiseCnt); % noise reduction
        % morphological processing
        img_seg  = imfill(img_seg,'holes');
        img_seg  = imclose(img_seg,se);
        
        %% circle detection
        ObjectsStats = regionprops(img_seg, 'Area', 'Perimeter', 'BoundingBox');
        ObjectsCnt   = length(ObjectsStats);
        CirclesCnt   = 0;
        for i = 1:ObjectsCnt
            CompactnessValue = ObjectsStats(i).Perimeter.^2 / ObjectsStats(i).Area;
            if (CompactnessValue < CircleValThr)
                CirclesCnt = CirclesCnt + 1;
            end;
        end;
    else % static scene without any object
        ObjectDetected = 0;
    end;
    
    %% show results
    subplot(2,3,1), imshow(img_model), title('Background model','Color',TextColor,'FontSize',FontSize);
    subplot(2,3,2), imshow(img), title(['Current image: ',num2str(ImgCnt),', FPS: ',num2str(round(ImgCnt/toc))],'Color',TextColor,'FontSize',FontSize);
    subplot(2,3,3), imshow(img_diff), title(['MotionCoef: ',num2str(sum_motion,'%.1f'),', DiffCoef: ',num2str(sum_diff,'%.1f')],'Color',TextColor,'FontSize',FontSize);
    if (ObjectDetected)
        subplot(2,3,4), imshow(img_bw), title(['Binary image (threshold: ',num2str(ThrLevel),')'],'Color',TextColor,'FontSize',FontSize);
        subplot(2,3,5), imshow(img_seg), title(['Segmented objects: ',num2str(ObjectsCnt)],'Color',TextColor,'FontSize',FontSize);
        if (ObjectsCnt)
            subplot(2,3,6), imshow(img), title(['Detected circles: ',num2str(CirclesCnt)],'Color',TextColor,'FontSize',FontSize);
            for i = 1:ObjectsCnt
                CompactnessValue = (ObjectsStats(i).Perimeter).^2 / ObjectsStats(i).Area;
                if (CompactnessValue < CircleValThr)
                    x1 = ObjectsStats(i).BoundingBox(1);
                    y1 = ObjectsStats(i).BoundingBox(2);
                    x2 = x1 + ObjectsStats(i).BoundingBox(3);
                    y2 = y1 + ObjectsStats(i).BoundingBox(4);
                    % classification
                    if (ObjectsStats(i).Area>80 && ObjectsStats(i).Area<120)
                        CircleColor = [0 1 0];
                    else
                        CircleColor = [1 0 0];
                    end;
                    line([x1 x2 x2 x1 x1],[y1 y1 y2 y2 y1],'Color',CircleColor);
                    text(x1,y1-10,['A:',num2str(ObjectsStats(i).Area),', P:',num2str(uint32(ObjectsStats(i).Perimeter))],'Color',CircleColor,'FontSize',FontSize);
                end;
            end;
        end;
    else
        img_blank = zeros(ImgSize(2),ImgSize(1),'uint8');
        subplot(2,3,4), imshow(img_blank), line([1 ImgSize(1)],[1 ImgSize(2)],'Color',DrawColor), line([ImgSize(1) 1],[1 ImgSize(2)],'Color',DrawColor);
        subplot(2,3,5), imshow(img_blank), line([1 ImgSize(1)],[1 ImgSize(2)],'Color',DrawColor), line([ImgSize(1) 1],[1 ImgSize(2)],'Color',DrawColor);
        subplot(2,3,6), imshow(img_blank), line([1 ImgSize(1)],[1 ImgSize(2)],'Color',DrawColor), line([ImgSize(1) 1],[1 ImgSize(2)],'Color',DrawColor);
    end;
    drawnow;
    
end;
stop(cam);
close all;


function MyKeyPressed(src, event) %#ok<INUSL>
%% key press
global AppRunning;
switch event.Key
    case 'escape'
        AppRunning = 0;
    otherwise
        AppRunning = 0;
end;
